import xbmcgui
import xbmcplugin

from resources.lib.const import LANG, ROUTE, SETTINGS
from resources.lib.routing.router import router
from resources.lib.utils.kodiutils import get_string, ADDON
from resources.lib.storage.settings import settings


class DirectoryItem(object):
    TITLE = ''
    DIRECTORY = True
    ICON = ''
    URL = ''

    def __init__(self, title='', url='', icon=''):
        self._title = title or self.TITLE
        self._url = url or self.URL
        self._icon = icon or self.ICON

    def __call__(self, handle, *args, **kwargs):
        url, li, is_folder = self.build()
        xbmcplugin.addDirectoryItem(handle, url, li, isFolder=is_folder)

    def build(self):
        inner_item = xbmcgui.ListItem(
            label=self._title,
            path=self._url,
        )
        inner_item.setArt({
            'icon': self._icon,
        })

        return self._url, inner_item, self.DIRECTORY,


class WatchHistoryItem(DirectoryItem):
    ICON = 'DefaultInProgressShows.png'
    TITLE = get_string(LANG.WATCH_HISTORY)


class SearchItem(DirectoryItem):
    ICON = 'DefaultAddonsSearch.png'
    TITLE = get_string(30204)


class MoviesItem(DirectoryItem):
    ICON = 'DefaultMovies.png'
    TITLE = get_string(30200)


class TvShowsItem(DirectoryItem):
    ICON = 'DefaultTvshows.png'
    TITLE = get_string(30201)


class MainMenuFolderItem(DirectoryItem):
    ICON = 'DefaultFolderBack.png'
    TITLE = get_string(30202)


class SeasonItem(DirectoryItem):
    ICON = 'DefaultTVShowTitle.png'


class SettingsItem(DirectoryItem):
    ICON = 'DefaultAddonService.png'
    TITLE = get_string(30208)
    DIRECTORY = False


is_playable = {
    False: 'false',
    True: 'true',
}


class MediaItem(object):
    DIRECTORY = False

    def __init__(self, title, playable=True, url=None, art=None, cast=None, info_labels=None, stream_info=None,
                 services=None):
        self._title = title
        self._url = url
        self._art = art
        self._cast = cast
        self._info_labels = info_labels
        self._stream_info = stream_info
        self._services = services
        self._cast = cast
        self._playable = playable

    def __call__(self, handle):
        url, li, is_folder = self.build()
        xbmcplugin.addDirectoryItem(handle, url, li, isFolder=is_folder)

    @property
    def title(self):
        return self._title

    def build(self):
        """Creates the ListItem together with metadata."""
        item = xbmcgui.ListItem(
            label=self._title,
            path=self._url,
        )
        menu = []
        del self._info_labels['parent_titles']
        item.setArt(self._art)
        item.setInfo('video', self._info_labels)
        if self._stream_info:
            menu.append((get_string(30404), 'PlayMedia({}?force=true)'.format(self._url)))
            for key, value in self._stream_info.items():
                item.addStreamInfo(key, value)
        if self._services:
            for service in ['tvdb', 'imdb', 'tmdb']:
                item.setUniqueIDs({service: str(self._services.get(service))}, 'imdb')

        item.setCast(self._cast)
        item.setProperty('IsPlayable', is_playable[self._playable])
        if settings[SETTINGS.MOVIE_LIBRARY_FOLDER] != '':
            menu.append((get_string(LANG.ADD_TO_LIBRARY), "RunPlugin(%s)" % router.get_url(ROUTE.ADD_TO_LIBRARY)))

        item.addContextMenuItems(menu)
        return self._url, item, self.DIRECTORY,


class ParentItem(MediaItem):
    DIRECTORY = True


class TvShowItem(MediaItem):
    def __init__(self, *args, **kwargs):
        super(TvShowItem, self).__init__(*args, **kwargs)


class InfoDialogType:
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'


class InfoDialog:
    def __init__(self, message, heading=ADDON.getAddonInfo('name'), icon='', time=3000, sound=False):
        self._heading = heading
        self._message = message
        self._icon = InfoDialog._get_icon(icon)
        self._time = time
        self._sound = sound

    @staticmethod
    def _get_icon(icon):
        if icon == '':
            return ADDON.getAddonInfo('icon')
        elif icon == InfoDialogType.INFO:
            return xbmcgui.NOTIFICATION_INFO
        elif icon == InfoDialogType.WARNING:
            return xbmcgui.NOTIFICATION_WARNING
        elif icon == InfoDialogType.ERROR:
            return xbmcgui.NOTIFICATION_ERROR

    def notify(self):
        xbmcgui.Dialog().notification(self._heading, self._message, self._icon, self._time, sound=self._sound)
